# from rsshub.spiders.zhihu.explore import ctx
# from rsshub.spiders.zhihu.daily import ctx
# from rsshub.spiders.kr36.rss import ctx
from rsshub.spiders.tadoku.books import ctx
# from rsshub.spiders.ruanyifeng.rss import ctx
# from rsshub.spiders.v2x.rss import ctx
# from rsshub.spiders.gscore.rss import ctx
# from rsshub.spiders.bmpi.rss import ctx

from pprint import pprint

def t():
    a = ctx()
    pprint(a)


t()
